function PATHS=irreducible_paths(A,src,trgt,MAX_BRANCH_COST)
% Computes irreducible paths given a graph for BGA Patrolling Algorithm.
% basilico@elet.polimi.it, AIRLab, DEI, PoliMI
%
% INPUT: 
% A: Adjacency binary matrix describing the graph undirected topology;
% if using a grid this is the naming convention:
% 1 - 2 - 3
% |   |   |
% 4 - 5 - 6
% |   |   |
% 7 - X - 8
% this graph is described by a 9x9 matrix, the edge between 5 and 6 is
% given by A(5,6)=1, X is an obstacle cell
% src: the source node
% trgt: the target node
% MAX_BRANCH_COST: the maximun number of hops for a path
%%
% OUTPUT:
% PATHS: 
% the matrix of paths:
% | path1 |
% | path2 |
% | path3 |
% where:
% pathn = [src n2 n3 ... trgt 0 ... 0]
% 0 is used to denote a closed branch on the tree of paths
% every path p in PATHS is irreducible, i.e.,
% ~EXISTS p' | p' SUBSET_OF p




% it's ok...
temp = trgt;
trgt = src;
src = temp;

if sum(diag(A))
    disp('WARNING: the adjacency matrix contains autoarcs.')
end
    
B = [trgt]; % tree branches
done = 0;
while ~done
    NEXT=[]; % branches after a one step expansion
    % ONE STEP EXPANSION OF EACH BRANCH
    for i=1:length(B(:,1)); % for every branch
        % this branch stuff
        br_length = length(B(1,:)); % this branch length (number of vertices)
        br_last = B(i,end); % this branch leaf
        % if the branch ends with 0 it has been previously closed
        if (br_last~=0)
            ex = find(A(br_last,:)); % find reachable vertices from the branch current leaf
        else
            ex = []; % if it's closed there's no legal successor
        end
                
        % close the branch if: 
        % no legal successors
        % maximum length has been reached
        % source has been reached
        if(isempty(ex) || br_length >= MAX_BRANCH_COST || br_last==src)
            ex=0;            
        end                
                
        if br_length>1 && ~(isempty(ex) || sum(ex) == 0)
            ex = setdiff(ex,B(i,:)); % no revisit
            if(isempty(ex))
                ex = 0;
            end
        end
                
        N = repmat(B(i,:),length(ex),1); % replicate the branch for every legal successor
        N(:,end+1)=ex; % append the legal successors
        NEXT=[NEXT;N];
    end
    B = NEXT; % New tree of paths after one step expansion
        
    if sum(B(:,end))==0 % all the branches are closed
        done = 1;
    end
end
    
removed = [];
for x=1:size(B,1)
    if last_node(B(x,:)) ~= src;
        removed = [removed x];
    end
end
B(removed,:) = [];   

% remove reducible paths
n_branches = length(B(:,1));
removed = [];
for outer=1:n_branches % for every path
    for inner=1:n_branches % for every path
        if inner == outer || ismember(inner,removed)
            continue;
        end
        p1 = B(outer,:);
        p1(p1==0)=[];
        p2 = B(inner,:);
        p2(p2==0)=[];
        p12 = intersect(p1,p2);
        if numel(p12) == numel(p2)
            if sort(p12) == sort(p2) % then p1 contains p2 and should be removed
                removed = [removed outer];
                break;
            end
        end
    end
end
B(removed,:) = [];
 
PATHS = B;